/***********************************************************************************
FILE NAME		bus_init.c
DESCRIPTION		Function to configure the external bus and memory.

Note - 
'#pragma section' used in this file instructs the linker to place all subsequent code
in the named section. This can be used to relocate code in the memory map. This
directive should be used when debugging code in RAM to provide a section name that
can be referenced in the build | toolchain | linker | section settings. Use of the
directive without the section name will reset the section to default. 

Copyright   : 2007 Renesas Technology Europe Ltd.
Copyright   : 2007 Renesas Technology Corporation.
All Rights Reserved.
***********************************************************************************/

/***********************************************************************************
Revision History
DD.MM.YYYY OSO-UID Description
23.02.2007 RTE-VNA First Release.
***********************************************************************************/

/***********************************************************************************
System Includes
***********************************************************************************/
/* 	This header file provides access to intrinsic functions useful for configuring
	and controlling the interrupts. */
#include <machine.h>

/***********************************************************************************
User Includes
***********************************************************************************/
/*	Following header file provides a structure to access on-chip I/O registers. */
#include "iodefine.h"
/*	Following header file provides prototype for the functions contained in this
	file. */
#include "bus_init.h"

/***********************************************************************************
Function Prototypes
***********************************************************************************/
static void init_puram_section(void);

/***********************************************************************************
User Program Code
***********************************************************************************/

/***********************************************************************************
Function Name	: ConfigureBusAndMemory
Description		: Configures the external bus and memory.
Parameters		: none.
Return value	: none.
***********************************************************************************/
void ConfigureBusAndMemory(void)
{
	/* Configure the CS0 space.	*/
	init_bsc_cs0();

	/* To use the SDRAM in clock mode 0 or 1, AC characteristics switching register
		must be initialized	*/
#ifdef USE_CLKMODE0
	init_puram_section();
	set_acswr();
#endif

	/* Configure CS3 space for SDRAM */
#ifndef USE_32BIT_SDRAM_INTERFACE
	init_sdram();
#else
	init_sdram32();	
#endif

    /* Configure cache */
#ifdef USE_CACHE_MEMORY	
	init_cache();
#endif
}
/**********************************************************************************
End of function ConfigureBusAndMemory
***********************************************************************************/

/***********************************************************************************
Function Name	: init_bsc_cs0
Description		: Initializes the CS0 space.
Parameters		: none.
Return value	: none.
***********************************************************************************/
void init_bsc_cs0(void)
{
	/* Set to WEn output */
	PORT.PCCRL4.BIT.PC13MD = 1;

	/* Output A21 as we need 4MB space. */
	PORT.PECRL1.BIT.PE2MD = 1; 

	/* Disable BREQ as this is not used. Keep other settings as default. */
	BSC.CMNCR.LONG = 0x00001810UL;
		
	/* Configure CS0 space bus control	
	  1 Idle Cycles between Write-Read Cycles and Write-Write Cycles
	  0 Idle Cycles for Another Space Read-Write 
	  0 Idle Cycles for Read-Write in the Same Space
	  0 Idle Cycles for Read-Read in Another Space
	  0 Idle Cycles for Read-Read in the Same Space
	  Type of memory connected to a space - Normal
	  Arranged in big endian
	  16 bit Data Bus Width   */
	  
	BSC.CS0BCR.LONG = 0x10000400UL;

	/* Configure CS0 wait control register
	Number of Delay Cycles from Address, CS0 Assertion to RD, WEn Assertion - 1.5 cycles
	Number of Access Wait Cycles - 6 cycles
	External wait input is ignored
	Delay Cycles from RD, WEn Negation to Address, CS0 Negation - 1.5 cycles */
	
	BSC.UN0_BSC.NORMAL.REG_CS0WCR.LONG = 0x00000B41UL;
}
/**********************************************************************************
End of function init_bsc_cs0
***********************************************************************************/

#ifdef USE_32BIT_SDRAM_INTERFACE
/***********************************************************************************
Function Name	: init_sdram32
Description		: Initialize 32 bit RAM interface
Parameters		: none.
Return value	: none.
***********************************************************************************/
void init_sdram32(void)
{	
	/* 200usec wait count @200MHz */
	volatile int j = 40000;			
	
	/* Configure the port pins connected to the SDRAM interface	*/
	/* Configure the PC4 pin as DQMLL */
	PORT.PCCRL2.BIT.PC4MD = 1;	
	/* Configure the PC5 pin as DQMLU */
	PORT.PCCRL2.BIT.PC5MD = 1;	
	/* Configure the PC6 pin as DQMUL */
	PORT.PCCRL2.BIT.PC6MD = 1;	
	/* Configure the PC7 pin as DQMUU */
	PORT.PCCRL2.BIT.PC7MD = 1;	
	/* Configure the PC12 pin as CKE  */
	PORT.PCCRL4.BIT.PC12MD = 1;	
	/* Configure the PC13 pin as RD/WRn */
	PORT.PCCRL4.BIT.PC13MD = 1;	
	/* Configure the PC3 pin as CS3 */
	PORT.PCCRL1.BIT.PC3MD = 1;	
	/* Configure the PC9 pin as CASL */
	PORT.PCCRL3.BIT.PC9MD = 1;	
	/* Configure the PC8 pin as RASL */
	PORT.PCCRL3.BIT.PC8MD = 1;	

	/* Configure the PD15 pin as D31 */
	PORT.PDCRL4.BIT.PD15MD = 1;	
	/* Configure the PD15 pin as D30 */
	PORT.PDCRL4.BIT.PD14MD = 1;	
	/* Configure the PD15 pin as D29 */
	PORT.PDCRL4.BIT.PD13MD = 1;	
	/* Configure the PD15 pin as D28 */
	PORT.PDCRL4.BIT.PD12MD = 1;	
	/* Configure the PD15 pin as D27 */
	PORT.PDCRL3.BIT.PD11MD = 1;	
	/* Configure the PD15 pin as D26 */
	PORT.PDCRL3.BIT.PD10MD = 1;	
	/* Configure the PD15 pin as D25 */
	PORT.PDCRL3.BIT.PD9MD = 1;	
	/* Configure the PD15 pin as D24 */
	PORT.PDCRL3.BIT.PD8MD = 1;	
	/* Configure the PD15 pin as D23 */
	PORT.PDCRL2.BIT.PD7MD = 1;	
	/* Configure the PD15 pin as D22 */
	PORT.PDCRL2.BIT.PD6MD = 1;	
	/* Configure the PD15 pin as D21 */
	PORT.PDCRL2.BIT.PD5MD = 1;	
	/* Configure the PD15 pin as D20 */
	PORT.PDCRL2.BIT.PD4MD = 1;	
	/* Configure the PD15 pin as D19 */
	PORT.PDCRL1.BIT.PD3MD = 1;	
	/* Configure the PD15 pin as D18 */
	PORT.PDCRL1.BIT.PD2MD = 1;	
	/* Configure the PD15 pin as D17 */
	PORT.PDCRL1.BIT.PD1MD = 1;	
	/* Configure the PD15 pin as D16 */
	PORT.PDCRL1.BIT.PD0MD = 1;	

	/* Configure CS3 address space
	   Idle Cycles between Write-read Cycles and Write-write Cycles : 0 idle cycles 
	   Memory type : SDRAM 
	   Data Bus Size : 32-bit size 	*/
	BSC.CS3BCR.LONG = 0x00004600UL;	
									
	/* Configure CS3 wait control register
	Number of Auto-Precharge Completion Wait Cycles - 1 cycle
	Number of Wait Cycles between ACTV Command and READ(A)/WRIT(A) Command - 2 cycles
	CAS Latency - 3 cycles
	Number of Access Wait Cycles - 5 cycles
	External wait input is ignored  */
	
	BSC.UN2_BSC.SDRAM.REG_CS3WCR.LONG = 0x00002912UL;

	/* 	Configure SDRAM control register
		Refresh Control :Refresh 
		RMODE :Auto-refresh is performed 
		BACTV :Auto-precharge mode 
		Row address for Area3 :12-bits 
		Column Address for Area3 :9-bits 	*/
	
	BSC.SDCR.LONG = 0x00000809UL;	

	/* Configure Refresh time constant register 
		15.625usec /240nsec = 64 (0x41)cycles per refresh */

	BSC.RTCOR.LONG = 0xa55a0041UL;	

	/* Configure Refresh timer control/status register 
       Initialization sequence start Clock select B-phy/16 = 240nsec
	   Refresh count :Once  */

	BSC.RTCSR.LONG = 0xa55a0010UL;	
									
	while(j-- > 0)
	{
		/* Wait for 200us interval */
	}

	/* Configure SDRAM Mode Register for 32 bit interface */
	SDRAM_MODE32 = 0;				
}
/**********************************************************************************
End of function init_sdram32
***********************************************************************************/
#else
/***********************************************************************************
Function Name	: init_sdram
Description		: Configures the CS3 space for SDRAM
Parameters		: none.
Return value	: none.
***********************************************************************************/
void init_sdram(void)
{
	/* 200usec wait count @200MHz */
	volatile int j = 40000;			
	
	/* Configure the port pins connected to the SDRAM interface	*/
	/* Configure the PC4 pin as DQMLL */
	PORT.PCCRL2.BIT.PC4MD = 1;	
	/* Configure the PC5 pin as DQMLU */
	PORT.PCCRL2.BIT.PC5MD = 1;	
	/* Configure the PC6 pin as DQMUL */
	PORT.PCCRL2.BIT.PC6MD = 1;	
	/* Configure the PC7 pin as DQMUU */
	PORT.PCCRL2.BIT.PC7MD = 1;	
	/* Configure the PC12 pin as CKE  */
	PORT.PCCRL4.BIT.PC12MD = 1;	
	/* Configure the PC13 pin as RD/WRn */
	PORT.PCCRL4.BIT.PC13MD = 1;	
	/* Configure the PC3 pin as CS3 */
	PORT.PCCRL1.BIT.PC3MD = 1;	
	/* Configure the PC9 pin as CASL */
	PORT.PCCRL3.BIT.PC9MD = 1;	
	/* Configure the PC8 pin as RASL */
	PORT.PCCRL3.BIT.PC8MD = 1;	

	/* Configure CS3 space bus control	
	  2 Idle Cycles between Write-Read Cycles and Write-Write Cycles
	  2 Idle Cycles for Another Space Read-Write 
	  2 Idle Cycles for Read-Write in the Same Space
	  2 Idle Cycles for Read-Read in Another Space
	  2 Idle Cycles for Read-Read in the Same Space
	  Type of memory connected to a space - SDRAM
	  Arranged in big endian
	  32 bit Data Bus Width   */

	BSC.CS3BCR.LONG = 0x00004400UL;	
									
	/* Configure CS3 wait control register
	Number of Auto-Precharge Completion Wait Cycles - 1 cycle
	Number of Wait Cycles between ACTV Command and READ(A)/WRIT(A) Command - 2 cycles
	CAS Latency - 3 cycles
	Number of Access Wait Cycles - 5 cycles
	External wait input is ignored  */
	
	BSC.UN2_BSC.SDRAM.REG_CS3WCR.LONG = 0x00002912UL;

	/* 	Configure SDRAM control register
		Refresh Control :Refresh 
		RMODE :Auto-refresh is performed 
		BACTV :Auto-precharge mode 
		Row address for Area3 :12-bits 
		Column Address for Area3 :9-bits 	*/
	
	BSC.SDCR.LONG = 0x00000809UL;	

	/* Configure Refresh time constant register 
		15.625usec /240nsec = 64 (0x41)cycles per refresh */
	
	BSC.RTCOR.LONG = 0xa55a0041UL;	

	/* Configure Refresh timer control/status register 
       Initialization sequence start Clock select B-phi/16 = 240nsec
	   Refresh count : Once  */

	BSC.RTCSR.LONG = 0xa55a0010UL;	

	while(j-- > 0)
	{
		/* Wait for 200us interval */
	}

	/* Configure SDRAM Mode Register for 16 bit interface */
	SDRAM_MODE = 0;					
}
/**********************************************************************************
End of function init_sdram
***********************************************************************************/
#endif

/***********************************************************************************
Function Name	: init_puram_section
Description		: Copies set_acswr() function to internal RAM. Please refer to 
				  the description of set_acswr() function for more details.
Parameters		: none.
Return value	: none.
***********************************************************************************/
static void init_puram_section(void)
{
	unsigned long *src,*end,*dst;

	/* Initialize the pointers	*/
	src = (unsigned long *)__sectop("PURAM");
	end = (unsigned long *)__secend("PURAM");
	dst = (unsigned long *)__sectop("RPURAM");
	
	/* Copy the whole section */
	while(src < end)
	{
		*dst++ = *src++;
	}
}
/***********************************************************************************
End of function init_puram_section
***********************************************************************************/

/***********************************************************************************
Function Name	: set_acswr
Description		: Configure AC Characteristics Switching Registers. 
				  This function MUST be executed in internal RAM. To make this possible,
				  init_puram_section() function copies the section from ROM section 
				  "PURAM" to RAM section "RPURAM".
				  Please note that this is only required if the CPU is running in
				  Clock mode 0 or clock mode 1.
Parameters		: none.
Return value	: none.
***********************************************************************************/
#pragma section URAM
void set_acswr (void)
{
	do
	{
		/* Configure AC Characteristics Switch setting */
		AC.ACKEYR.BYTE = 0;			
		AC.ACKEYR.BYTE = 0;
		AC.ACSWR.LONG = 0x9UL;		
	} while(AC.ACSWR.LONG != 0x9UL);
}
#pragma section
/***********************************************************************************
End of function set_acswr
***********************************************************************************/

#ifdef USE_CACHE_MEMORY	
/***********************************************************************************
Function Name	: init_cache
Description		: Configure the cache memory
Parameters		: none.
Return value	: none.
***********************************************************************************/
//#pragma section CACHE		
void init_cache(void)
{ 
	volatile unsigned long reg;
	int mask;

	/* Read & store current interrupt mask setting */
	mask = get_imask();
	
	/* Mask all interrupts */
	set_imask(0x0F);					

	/* Configure Cache register 
		ICF=1:Instruction cache flushed
		ICE=1:Instruction cache enabled
		OCF=1:Operand cache flushed
		OCE=1:Operand cache enabled		*/
	
#ifdef CACHE_WRITEBACK
	/* Write back ON */
	CCNT.CCR1.LONG = 0x0909UL;	
#else
	/* Write back OFF */
	CCNT.CCR1.LONG = 0x090BUL;	
#endif	

	/* Read back the cache register	*/
	reg = CCNT.CCR1.LONG ;

	/* Set to the original level of interrupt mask */
	set_imask(mask);					
}
/***********************************************************************************
End of function init_cache
***********************************************************************************/

/***********************************************************************************
Function Name	: cache_writeback
Description		: cache write back
Parameters		: none.
Return value	: none.
***********************************************************************************/
int cache_writeback(void)
{ 
	volatile unsigned long *arry;
	unsigned int i,j;
	int mask;

	/* Read & store current interrupt mask setting */
	mask = get_imask();
	
	/* Mask all interrupts */
	set_imask(0x0F);					
	
	/* Invalidate all entries */
	for(i=0u; i <4u; i++)
	{
		for(j=0u; j < 128u; j++)
		{
			/* Creating an address array address */
			arry = (volatile unsigned long *)(0xf0800000 | (i<<11) | (j<<4));

			/* Write U=0 and V=0 in the address array V=0,U=0  */
			*arry &= 0xfffffffcUL;		
		}
	}
	
	/* Set to the original level of interrupt mask */
	set_imask(mask);					
	
	return 0;
}
//#pragma section
/***********************************************************************************
End of function cache_writeback
***********************************************************************************/
#endif

/***********************************************************************************
End of file
***********************************************************************************/
