/***********************************************************************************
FILE NAME  		hwsetup.c
DESCRIPTION		hardware setup functions.
		This file contains the functions required to initialise the RSK just before
		calling main application.

Copyright   : 2007 Renesas Technology Europe Ltd.
Copyright   : 2007 Renesas Technology Corporation.
All Rights Reserved
***********************************************************************************/

/***********************************************************************************
Revision History
DD.MM.YYYY OSO-UID Description
23.02.2007 RTE-VNA First Release.
***********************************************************************************/

/***********************************************************************************
System Includes
***********************************************************************************/
/* 	This header file provides access to intrinsic functions useful for configuring
	and controlling the interrupts. */
#include <machine.h>

/***********************************************************************************
User Includes
***********************************************************************************/
/*	Following header file provides a structure to access on-chip I/O registers. */
#include "iodefine.h"
/*	Following header file provides prototype for hardware setup function. */
#include "hwsetup.h"

/***********************************************************************************
User Program Code Prototypes
***********************************************************************************/
/* These functions are private so their prototypes are defined locally */
static void ConfigureOperatingFrequency(void);
static void EnablePeripheralModules(void);
static void ConfigurePortPins(void);

/***********************************************************************************
User Program Code
***********************************************************************************/

/***********************************************************************************
Function Name : HardwareSetup
Description   :	Sets up the hardware.
		This function makes a call to initialisation functions to configure the CPU
		operating frequency, port pins & relevant on-chip modules (i.e. such as ADC,
		timer etc.) in order to setup the RSK for the main application.
Parameters    :	none
Return value  :	none
***********************************************************************************/
void HardwareSetup(void) {    
  ConfigureOperatingFrequency();
  ConfigureBusAndMemory();
  ConfigurePortPins();
  EnablePeripheralModules();
}
/***********************************************************************************
End of function HardwareSetup
***********************************************************************************/

/***********************************************************************************
Function Name : ConfigureOperatingFrequency
Description	  : Sets up CPU operating speed.
		This function configures the CPU operating frequency. 
Parameters	  : none
Return value  : none
***********************************************************************************/
void ConfigureOperatingFrequency(void) {
   /* Stop WDT by writing 0 to all un-reserved bits with
       the upper byte containing H'A5 */
    WDT.WTCSR.WORD = 0xA518;

    /* The peripheral clock is 25MHz at start-up. To make a 10mS clock
       settling delay it must be set to count 100,000 cycles (10mS / (1 / 12.5 MHz)).
       To do this the peripheral clock is divided by 1024 (CKS = B'101 or H'5),
       and the WTCNT register is loaded with 132 (255 - (125,000 / 1024)).
       First set the peripheral clock divisor to H'5 for division by 1024 */
    WDT.WTCSR.WORD = 0xA51D;

    /* Set WCTNT to 132 to get an overflow period of 10ms */
    WDT.WTCNT.WORD = (0x5A00 + 132);

	/* For clock mode 0 */	
	/*	FRQCR setting for I=XTAL*16 B=XTAL*4 P=XTAL*2 */
	/*	B15:B14 - Reserved - set to 0
		B13:B12 - Clock Output Enable - set to 0
		B11:B10 - Reserved - set to 0
		B9:B8 - Frequency multiplication ratio of PLL circuit 1 - 16 times
		B7:B5 - Reserved - Set to 0
		B4 - Internal Clock (Iphi) Frequency Division Ratio - 1 time
		B3 - Reserved - Set to 0
		B2:B0 - Peripheral Clock (Pphi) Frequency Division Ratio - 1/8 time	*/

	CPG.FRQCR.WORD = 0x0104;
}
/***********************************************************************************
End of function ConfigureOperatingFrequency
***********************************************************************************/

/***********************************************************************************
Function Name : ConfigurePortPins
Description	  : Configures port pins of the MCU.
		This function configures the port pins so that it can be used by the 
		respective modules. It also configures the unused port pins as output low.
Parameters	  : none.
Return value  : none.
***********************************************************************************/
void ConfigurePortPins(void) {
/* Port pins default to inputs. To ensure safe initialisation set the pin states
before changing the data direction registers. This will avoid any unintentional 
state changes on the external ports.
Many peripheral modules will override the setting of the port registers. Ensure that
the state is safe for external devices if the internal peripheral module is disabled
or powered down. */

}
/***********************************************************************************
End of function ConfigurePortPins
***********************************************************************************/

/***********************************************************************************
Function Name : EnablePeripheralModules
Description   : Enables Peripheral Modules before use.
		This function enables the on-chip modules used by the main application.
Parameters	  : none
Return value  : none
***********************************************************************************/
void EnablePeripheralModules(void) {
}
/***********************************************************************************
End of function EnablePeripheralModules
***********************************************************************************/
