/******************************************************************************
* DISCLAIMER:
* The software supplied by Renesas Technology Europe Ltd is
* intended and supplied for use on Renesas Technology products.
* This software is owned by Renesas Technology Europe, Ltd. Or
* Renesas Technology Corporation and is protected under applicable
* copyright laws. All rights are reserved.
*
* THIS SOFTWARE IS PROVIDED "AS IS". NO WARRANTIES, WHETHER EXPRESS,
* IMPLIED OR STATUTORY, INCLUDING BUT NOT LIMITED TO IMPLIED
* WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
* APPLY TO THIS SOFTWARE. RENESAS TECHNOLOGY AMERICA, INC. AND
* AND RENESAS TECHNOLOGY CORPORATION RESERVE THE RIGHT, WITHOUT
* NOTICE, TO MAKE CHANGES TO THIS SOFTWARE. NEITHER RENESAS
* TECHNOLOGY AMERICA, INC. NOR RENESAS TECHNOLOGY CORPORATION SHALL,
* IN ANY CIRCUMSTANCES, BE LIABLE FOR SPECIAL, INCIDENTAL, OR
* CONSEQUENTIAL DAMAGES FOR ANY REASON WHATSOEVER ARISING OUT OF THE
* USE OR APPLICATION OF THIS SOFTWARE.
******************************************************************************/

/* Copyright (C) 2009. Renesas Technology Europe, All Rights Reserved */

/*FILE COMMENT******************************* Technical reference data ****
* File Name		: hwsetup.c
* Version 		: 1.00
* Device 		: SH/7264 (R5S72643W144FPU)
* Tool Chain 	: HEW, Renesas SuperH Standard Tool chain v9.3
* H/W Platform	: RSK2+SH7264
* Description 	: Contains hardware setup functions
******************************************************************************/

/******************************************************************************
* History 		: 12.11.2009 Ver. 1.00 First Release
*FILE COMMENT END*********************************************************/

/******************************************************************************
User Includes (Project level includes)
******************************************************************************/
/*	Following header file provides a structure to access on-chip I/O 
	registers. */	
#include "iodefine.h"
/* Following header file provides common defines for widely used items. */
#include "rsksh7264def.h"
/* Following header file provides prototypes for hardware configuration
   functions. */
#include "hwsetup.h"
/* Following header file provides prototypes for external bus initialization 
   and cache initialization functions. */
#include "bus_init.h"

/******************************************************************************
User Program Code Prototypes
******************************************************************************/
/* These functions are private so their prototypes are defined locally */
static void ConfigureOperatingFrequency(void);
static void EnablePeripheralModules(void);
static void ConfigurePortPins(void);

/******************************************************************************
User Program Code
******************************************************************************/

/*FUNC COMMENT*************************************************************
* Outline 		: HardwareSetup
* Description 	: Sets up the hardware.
*   			  This function calls the hardware initialization functions to
*				  configure the CPU operating frequency, port pins & required
*				  on-chip modules in order to setup the RSK for the main 
*				  application.
* Argument  	: none
* Return value  : none
*FUNC COMMENT END*********************************************************/

void HardwareSetup(void)
{
	/* Configures CPU clock	*/
	ConfigureOperatingFrequency();

	/* configure the external Memory */
	ConfigureBusAndMemory();

	/* Configures port pins	*/
	ConfigurePortPins();

	/* Enables required on-chip peripherals	*/
	EnablePeripheralModules();
}
/******************************************************************************
End of function HardwareSetup
******************************************************************************/

/*FUNC COMMENT*************************************************************
* Outline 		: ConfigureOperatingFrequency
* Description 	: Sets up operating speed and configures the main & sub-clock
*				  and USB clock	
* Argument  	: none
* Return value  : none
*FUNC COMMENT END*********************************************************/

static void ConfigureOperatingFrequency(void)
{
	/*	Configure Frequency Control Register
		b2:b0	- PFC2:PFC0		- set to 011 (pclk = internal clock /4) 
		b3		- Reserved 		- Set to 0
		b5:b4 	- IFC1:IFC0		- No frequency division
		b7:b6 	- Reserved 		- Set to 00
		b8 		- STC			- 8 times (with clock mode 2)
		b11:b9	- Reserved 		- set to 000
		b13:b12	- CKOEN1:CKOEN0	- set to 01
		b14		- CKOEN2	 	- set to 0 
		b15		- Reserved 		- set to 0
		
		Multiplication Ratio I:B:P= 8:4:2 
		CKIO:Output 
		Clockin = 18 MHz, CKIO = 72 MHz 
		I Clock = 144 MHz, B Clock = 72 MHz, 
		P Clock = 36 MHz	*/
		
	CPG.FRQCR.WORD = 0x1003u;		
}
/******************************************************************************
End of function ConfigureOperatingFrequency
******************************************************************************/

/*FUNC COMMENT*************************************************************
* Outline 		: ConfigurePortPins
* Description 	: Configures MCU port pins. This function configures the port 
*				  pins so that it can be used by the respective modules. It 
*				  also configures the unused port pins as output low.
* Argument 		: none
* Return value 	: none
*FUNC COMMENT END*********************************************************/

void ConfigurePortPins(void)
{
	/* Configure LCD Pins */
	/*Configure PJ6 to PJ11 as IO port pins */
	PORT.PJCR1.BIT.PJ6MD = 0;
	PORT.PJCR1.BIT.PJ7MD = 0;
	PORT.PJCR2.BIT.PJ8MD = 0;
	PORT.PJCR2.BIT.PJ9MD = 0;
	PORT.PJCR2.BIT.PJ10MD = 0;
	PORT.PJCR2.BIT.PJ11MD = 0;

	/*Configure PJ6 to PJ11 as output */
	PORT.PJIOR0.BIT.PJ6IOR = 1;
	PORT.PJIOR0.BIT.PJ7IOR = 1;
	PORT.PJIOR0.BIT.PJ8IOR = 1;
	PORT.PJIOR0.BIT.PJ9IOR = 1;
	PORT.PJIOR0.BIT.PJ10IOR = 1;
	PORT.PJIOR0.BIT.PJ11IOR = 1;

	/* Configure AN0-AN3 and AN6 pins */
	PORT.PHCR0.BIT.PH0MD = 1;
	PORT.PHCR0.BIT.PH1MD = 1;
	PORT.PHCR0.BIT.PH2MD = 1;
	PORT.PHCR0.BIT.PH3MD = 1;
	PORT.PHCR1.BIT.PH6MD = 1;
}
/******************************************************************************
End of function ConfigurePortPins
******************************************************************************/

/*FUNC COMMENT*************************************************************
* Outline 		: EnablePeripheralModules
* Description 	: Enables Peripheral Modules before use 
* Argument  	: none
* Return value  : none
* Note			: When writing to the register related to power-down modes,  
* 				  the CPU, after executing a write instruction, executes the  
* 				  next instruction without waiting for the write operation to  
*				  complete. Therefore, to reflect the change specified by  
* 				  writing to the register while the next instruction is executed, 
* 				  insert a dummy read of the same register between the register 
*				  write instruction and the next instruction. 
*FUNC COMMENT END*********************************************************/

static void EnablePeripheralModules(void)
{
	volatile unsigned char dummy;
	
	/* Enable on-chip RAM */
	CPG.SYSCR1.BYTE = 0xFF;	
	dummy = CPG.SYSCR1.BYTE;
}
/******************************************************************************
End of function EnablePeripheralModules
******************************************************************************/
