/***********************************************************************************
FILE NAME		bus_init.c
DESCRIPTION		Function to configure the external bus and memory.

Note - 
'#pragma section' used in this file instructs the linker to place all subsequent code
in the named section. This can be used to relocate code in the memory map. This
directive should be used when debugging code in RAM to provide a section name that
can be referenced in the build | toolchain | linker | section settings. Use of the
directive without the section name will reset the section to default. 

Copyright   : 2008 Renesas Technology Europe Ltd.
Copyright   : 2008 Renesas Technology Corporation.
All Rights Reserved.
***********************************************************************************/

/***********************************************************************************
Revision History
DD.MM.YYYY OSO-UID Description
11.04.2008 RTE-VNA First Release.
***********************************************************************************/

/***********************************************************************************
System Includes
***********************************************************************************/
/* 	This header file provides access to intrinsic functions useful for configuring
	and controlling the interrupts. */
#include <machine.h>

/***********************************************************************************
User Includes
***********************************************************************************/
/*	Following header file provides a structure to access on-chip I/O registers. */
#include "iodefine.h"
/* Following header file provides common defines for widely used items. */
#include "rsk7670def.h"
/*	Following header file provides prototype for the functions contained in this
	file. */
#include "bus_init.h"

/***********************************************************************************
Function Prototypes
***********************************************************************************/
static void Init_Puram_Section(void);

/***********************************************************************************
User Program Code
***********************************************************************************/

/***********************************************************************************
Function Name	: ConfigureBusAndMemory
Description		: Configures the external bus and memory.
Parameters		: none.
Return value	: none.
***********************************************************************************/
void ConfigureBusAndMemory(void)
{
	/* Configure the CS0 space.	*/
	Init_BSC_CS0();

	/* To use the SDRAM in clock mode 0 or 1, AC characteristics switching register
		must be initialized	*/
#ifdef USE_CLKMODE0
	Init_Puram_Section();
	Set_ACSWR();
#endif

	/* Configure CS3 space for SDRAM */
	Init_SDRAM();

	/* Configure cache */
#ifdef USE_CACHE_MEMORY	
	Init_Cache();
#endif
}
/**********************************************************************************
End of function ConfigureBusAndMemory
***********************************************************************************/

/***********************************************************************************
Function Name	: Init_BSC_CS0
Description		: Initializes the CS0 space.
Parameters		: none.
Return value	: none.
***********************************************************************************/
void Init_BSC_CS0(void)
{
	/* Set A17-A23 */
	PORT.PACRH1.WORD = 0x5554;
		
	/* Configure CS0 space bus control	
	  1 Idle Cycles between Write-Read Cycles and Write-Write Cycles
	  0 Idle Cycles for Another Space Read-Write 
	  0 Idle Cycles for Read-Write in the Same Space
	  0 Idle Cycles for Read-Read in Another Space
	  0 Idle Cycles for Read-Read in the Same Space
	  Type of memory connected to a space - Normal
	  Arranged in big endian
	  Data Bus Width - select using MACRO  */

#ifdef USE_32BIT_FLASH_INTERFACE
	//BSC.CS0BCR.LONG = 0x10000600UL;
	BSC.CS0BCR.LONG = 0x10000400UL;
#else
	BSC.CS0BCR.LONG = 0x36DB0400UL;
#endif

	/* Configure CS0 wait control register
	Number of Delay Cycles from Address, CS0 Assertion to RD, WEn Assertion - 1.5 cycles
	Number of Access Wait Cycles - 6 cycles
	External wait input is ignored
	Delay Cycles from RD, WEn Negation to Address, CS0 Negation - 1.5 cycles */
	BSC.CS0WCR.LONG = 0x00000440UL;
}
/**********************************************************************************
End of function Init_BSC_CS0
***********************************************************************************/

/***********************************************************************************
Function Name	: Init_SDRAM
Description		: Configures the CS3 space for SDRAM
Parameters		: none.
Return value	: none.
***********************************************************************************/
void Init_SDRAM(void)
{
	/* 200usec wait count @200MHz */
	volatile int j = 40000;			
	
	/* Configure CS3 space bus control	
	  2 Idle Cycles between Write-Read Cycles and Write-Write Cycles
	  2 Idle Cycles for Another Space Read-Write 
	  2 Idle Cycles for Read-Write in the Same Space
	  2 Idle Cycles for Read-Read in Another Space
	  2 Idle Cycles for Read-Read in the Same Space
	  Type of memory connected to a space - SDRAM
	  Arranged in big endian
	  Data Bus Width - select using MACRO  */


	BSC.CS3BCR.LONG = 0x00004600UL;	

				
	/* Configure CS3 wait control register
	Number of Auto-Precharge Completion Wait Cycles - 1 cycle
	Number of Wait Cycles between ACTV Command and READ(A)/WRIT(A) Command - 2 cycles
	CAS Latency - 3 cycles
	Number of Access Wait Cycles - 5 cycles
	External wait input is ignored  */
	BSC.UN3_BSC.SDRAM.REG_CS3WCR.LONG = 0x00002912UL;

	/* 	Configure SDRAM control register
		Refresh Control :Refresh 
		RMODE :Auto-refresh is performed 
		BACTV :Auto-precharge mode 
		Row address for Area3 :12-bits 
		Column Address for Area3 :9-bits 	*/
	BSC.SDCR.LONG = 0x00000809UL;	

	/* Configure Refresh time constant register 
		15.625usec /160nsec = 98 (0x62)cycles per refresh */
	BSC.RTCOR.LONG = 0xa55a0062UL;	

	/* Configure Refresh timer control/status register 
       Initialization sequence start Clock select B-phi/16 = 160nsec
	   Refresh count : Once  */
	BSC.RTCSR.LONG = 0xa55a0010UL;	

	while(j-- > 0)
	{
		/* Wait for 200us interval */
	}

	/* Configure SDRAM Mode Register for 16 bit interface */
	SDRAM_MODE = 0;					
}
/**********************************************************************************
End of function Init_SDRAM
***********************************************************************************/

/***********************************************************************************
Function Name	: Init_Puram_Section
Description		: Initializes the RAM section
Parameters		: none.
Return value	: none.
***********************************************************************************/
static void Init_Puram_Section(void)
{
	unsigned long *src,*end,*dst;

	src = (unsigned long *) __sectop("PURAM");
	end = (unsigned long *) __secend("PURAM");
	dst = (unsigned long *) __sectop("RPURAM");
	
	while (src < end)
	{
		*dst++ = *src++;
	}
}
/***********************************************************************************
End of function Init_Puram_Section
***********************************************************************************/

/***********************************************************************************
Function Name	: Set_ACSWR
Description		: Configure AC Characteristics Switching Registers.
Parameters		: none.
Return value	: none.
***********************************************************************************/
#pragma section URAM
void Set_ACSWR (void)
{
/*	do
	{
		// Configure AC Characteristics Switch setting 
		AC.ACKEYR.BYTE = 0;			
		AC.ACKEYR.BYTE = 0;
		AC.ACSWR.LONG = 0x9UL;		
	}
	while (AC.ACSWR.LONG != 0x9UL);*/
}
#pragma section
/***********************************************************************************
End of function Set_ACSWR
***********************************************************************************/

#ifdef USE_CACHE_MEMORY	
/***********************************************************************************
Function Name	: Init_Cache
Description		: Configure the cache memory
Parameters		: none.
Return value	: none.
***********************************************************************************/
#pragma section CACHE		
void Init_Cache(void)
{ 
	volatile unsigned long reg;
	int mask;

	/* Read & store current interrupt mask setting */
	mask = get_imask();
	
	/* Mask all interrupts */
	set_imask(0x0F);					

	/* Configure Cache register 
		ICF=1:Instruction cache flushed
		ICE=1:Instruction cache enabled
		OCF=1:Operand cache flushed
		OCE=1:Operand cache enabled		*/
	
#ifdef CACHE_WRITEBACK
	/* Write back ON */
	CCNT.CCR1.LONG = 0x0909UL;
#else
	/* Write back OFF */
	CCNT.CCR1.LONG = 0x090BUL;	
#endif	

	/* Read back the cache register	*/
	reg = CCNT.CCR1.LONG ;

	/* Set to the original level of interrupt mask */
	set_imask(mask);					
}
/***********************************************************************************
End of function Init_Cache
***********************************************************************************/

/***********************************************************************************
Function Name	: Cache_Writeback
Description		: cache write back
Parameters		: none.
Return value	: none.
***********************************************************************************/
int Cache_Writeback(void)
{ 
	volatile unsigned long *arry;
	unsigned int i;
	unsigned int j;
	int mask;

	/* Read & store current interrupt mask setting */
	mask = get_imask();
	
	/* Mask all interrupts */
	set_imask(0x0F);					
	
	/* Invalidate all entries */
	for (i = 0U; i < 4U; i++)
	{
		for (j = 0U; j < 128U; j++)
		{
			/* Creating an address array address */
			arry = (volatile unsigned long*) (0xf0800000 | (i << 11) | (j << 4));

			/* Write U=0 and V=0 in the address array V=0,U=0  */
			*arry &= 0xfffffffcUL;		
		}
	}
	
	/* Set to the original level of interrupt mask */
	set_imask(mask);					
	
	return 0;
}
#pragma section
/***********************************************************************************
End of function Cache_Writeback
***********************************************************************************/
#endif
