/***********************************************************************************
FILE NAME  		hwsetup.c
DESCRIPTION		hardware setup functions.
		This file contains the functions required to initialise the RSK just before
		calling main application.

Copyright : 2008 Renesas Technology Europe Ltd.
Copyright : 2008 Renesas Technology Corporation.
All Rights Reserved
***********************************************************************************/

/***********************************************************************************
Revision History
DD.MM.YYYY OSO-UID Description
12.08.2008 RTE-ADE First Release.
***********************************************************************************/

/***********************************************************************************
System Includes
***********************************************************************************/
/* 	This header file provides access to intrinsic functions useful for configuring
	and controlling the interrupts. */
#include	<machine.h>

/***********************************************************************************
User Includes
***********************************************************************************/
/* Following header file provides a structure to access on-chip I/O registers. */
#include	"iodefine.h"
/* Following header file provides common defines for widely used items. */
#include	"rsk7670def.h"
/* Following header file provides prototype for hardware setup function. */
#include	"hwsetup.h"

/***********************************************************************************
User Defines
***********************************************************************************/
/* Remode this define when you use clock mode2 or mode3 */
#define		USE_CLKMODE0_1

/***********************************************************************************
User Program Code Prototypes
***********************************************************************************/
/* These functions are private so their prototypes are defined locally */
static void ConfigureOperatingFrequency(void);
static void EnablePeripheralModules(void);
static void ConfigurePortPins(void);
void ConfigureInterrupts(void);

/***********************************************************************************
User Program Code
***********************************************************************************/

/***********************************************************************************
Function Name:	HardwareSetup
Description:	Sets up the hardware.
		This function makes a call to initialisation functions to configure the CPU
		operating frequency, port pins & relevant on-chip modules (i.e. such as ADC,
		timer etc.) in order to setup the RSK for the main application.
Parameters:		none
Return value:	none
***********************************************************************************/
void HardwareSetup(void)
{
	ConfigureOperatingFrequency();
	ConfigureBusAndMemory();
	ConfigurePortPins();
	EnablePeripheralModules();
	//ConfigureInterrupts();
}
/***********************************************************************************
End of function HardwareSetup
***********************************************************************************/

/***********************************************************************************
Function Name : ConfigureOperatingFrequency
Description	  : Sets up CPU operating speed.
		This function configures the CPU operating frequency. 
Parameters	  : none
Return value  : none
***********************************************************************************/
void ConfigureOperatingFrequency(void)
{
	/* ==== CPG Setting ==== */
	/* WDT Clock select */
	/* 1/4096xP-phy 33.3MHz*/
	WDT.WTCSR.WORD = 0xa51e;
	/* Initial value of Counter: D'173 10mS */        
	WDT.WTCNT.WORD = 0x5aad;
	
	/* PLL1(x8),I:B:P=8:4:2
    CKIO:Output at time usually,Output when bus right is opened,output at standby"L"
    Clockin = 25  MHz, CKIO = 100 MHz
    I Clock = 200 MHz, B Clock = 100 MHz,
    P Clock = 50 MHz */
	CPG.FRQCR.WORD = 0x0003;        

	CPG.STBCR3.BYTE = 0x00;
	CPG.STBCR4.BYTE = 0x00;
}
/***********************************************************************************
End of function ConfigureOperatingFrequency
***********************************************************************************/

/***********************************************************************************
Function Name:	ConfigurePortPins
Description:	Configures port pins of the MCU.
		This function configures the port pins so that it can be used by the 
		respective modules. It also configures the unused port pins as output low.
Parameters:		none.
Return value:	none.
***********************************************************************************/
void ConfigurePortPins(void)
{
	/* To ensure safe initialisation set the pin states before changing the data
	direction registers. This will avoid any unintentional state changes on the
	external ports. Many peripheral modules will override the setting of the port
	registers. Ensure that the state is safe for external devices if the internal
	peripheral module is disabled or powered down. */
	
	/* Configure port pins connected to LEDs	*/
	PORT.PECRL1.BYTE.H &= 0xFC;
	PORT.PEIORL.BIT.PE4IOR = 1;
	PORT.PECRL1.BYTE.H &= 0xF3;
	PORT.PEIORL.BIT.PE5IOR = 1;
	PORT.PECRL2.BYTE.L &= 0xCF;
	PORT.PEIORL.BIT.PE10IOR = 1;
	PORT.PECRL2.BYTE.L &= 0x3F;
	PORT.PEIORL.BIT.PE11IOR = 1;

	/* LEDs OFF	*/
	LED0 = LED1 = LED2 = LED3 = LED_OFF;
/* Configure port pins connected to LCD	*/
// dlcdrs (a24)
	PORT.PADRH.BIT.PA24DR = SET_BIT_HIGH;
	PORT.PACRH2.BIT.PA24MD0 = SET_BIT_LOW;
	PORT.PAIORH.BIT.PA24IOR = SET_BIT_HIGH;

// dlcde
	PORT.PBDRL.BIT.PB4DR = SET_BIT_LOW;
	PORT.PBCRL1.BIT.PB4MD0 = SET_BIT_LOW;
	PORT.PBCRL1.BIT.PB4MD1 = SET_BIT_LOW;
	PORT.PBIORL.BIT.PB4IOR = SET_BIT_HIGH;

// 	d0
	PORT.PDDRL.BIT.PD0DR = SET_BIT_LOW;
	PORT.PDCRL1.BIT.PD0MD0 = SET_BIT_LOW;
	PORT.PDCRL1.BIT.PD0MD1 = SET_BIT_LOW;
	PORT.PDIORL.BIT.PD0IOR = SET_BIT_HIGH;

// d1
	PORT.PDDRL.BIT.PD1DR = SET_BIT_LOW;
	PORT.PDCRL1.BIT.PD1MD0 = SET_BIT_LOW;
	PORT.PDCRL1.BIT.PD1MD1 = SET_BIT_LOW;
	PORT.PDIORL.BIT.PD1IOR = SET_BIT_HIGH;

// d2
	PORT.PDDRL.BIT.PD2DR = SET_BIT_LOW;
	PORT.PDCRL1.BIT.PD2MD0 = SET_BIT_LOW;
	PORT.PDCRL1.BIT.PD2MD1 = SET_BIT_LOW;
	PORT.PDIORL.BIT.PD2IOR = SET_BIT_HIGH;

// d3
	PORT.PDDRL.BIT.PD3DR = SET_BIT_LOW;
	PORT.PDCRL1.BIT.PD3MD0 = SET_BIT_LOW;
	PORT.PDCRL1.BIT.PD3MD1 = SET_BIT_LOW;
	PORT.PDIORL.BIT.PD3IOR = SET_BIT_HIGH;


// serial port
	// SCIF2 TXD to pin.
	PORT.PECRL2.BIT.PE8MD0 = SET_BIT_LOW;
	PORT.PECRL2.BIT.PE8MD1 = SET_BIT_HIGH;
	PORT.PEIORL.BIT.PE8IOR = SET_BIT_HIGH;
	// SCIF2 RXD to pin.
	PORT.PECRL1.BIT.PE0MD0 = SET_BIT_LOW;
	PORT.PECRL1.BIT.PE0MD1 = SET_BIT_HIGH;
	PORT.PEIORL.BIT.PE0IOR = SET_BIT_LOW;
	

}
/***********************************************************************************
End of function ConfigurePortPins
***********************************************************************************/

/***********************************************************************************
Function Name:	EnablePeripheralModules
Description:	Enables Peripheral Modules before use.
		This function enables the on-chip modules used by the main application.
Parameters:		None
Return value:	None
***********************************************************************************/
void EnablePeripheralModules(void)
{
	/* All modules are active on reset */
}
/***********************************************************************************
End of function EnablePeripheralModules
***********************************************************************************/
